<?php
if(!defined('PLX_ROOT')) {
	exit;
}

class kzRating extends plxPlugin {

	const ADMIN_HOOKS = array(
		'AdminIndexFoot',
		'AdminCommentFoot',
	);

	const SITE_HOOKS = array(
		'kzRating',
		'ThemeEndBody',
		'plxMotorPreChauffageBegin',
		# 'plxShowConstruct',
	);

	const BEGIN_CODE = '<?php' . PHP_EOL;
	const END_CODE = PHP_EOL . '?>';
	const STARS_COUNT = 5;
	const WORKING_DIR = PLX_ROOT . PLX_CONFIG_PATH . 'plugins/' . __CLASS__;
	const XML_HEADER = '<?xml version="1.0" encoding="' . PLX_CHARSET . '"?>' . PHP_EOL;
	const QUERY_PATTERN = '@^article(\d{1,4})(?:/([\w-]+))?/' . __CLASS__ . '(\d{1,2})/([\da-f]{32})\b@';
	const FORMAT_NUM = '%0.1f';
	const SECONDS_PER_DAY = 24 * 3600;
	const POLLING_DURATION = 60 * self::SECONDS_PER_DAY; # 60 days

	const IP2LOCATION_PLUGIN_GITHUB_REPO = 'chrislim2888/IP2Location-PHP-Module';

	private $secret_key = '';
	private $limitPublished = '';

	public function __construct($lang) {

		parent::__construct($lang);
		if(!defined('PLX_ADMIN')) {
			foreach(self::SITE_HOOKS as $hook) {
				$this->addHook($hook, $hook);
			}
			$this->limitPublished = date('YmdHi', time() - self::POLLING_DURATION);
		} else {
			$this->setAdminProfil(PROFIL_ADMIN);
			foreach(self::ADMIN_HOOKS as $hook) {
				$this->addHook($hook, $hook);
			}

			if(isset($_SESSION['kzRating-plugin'])) {
				unset($_SESSION['kzRating-plugin']); # auto-nettoyage
			}
		}
	}

	public function onActivate() {
		if(!is_dir(self::WORKING_DIR)) {
			mkdir(self::WORKING_DIR);
		}
	}

	public function getStars($average) {
		for($i=1; $i<=self::STARS_COUNT; $i++) {
?>
				<li <?= ($i <= $average) ? 'class="on"' : '' ?> title="<?= $i ?> <?= $this->getLang(($i > 1) ? 'STARS' : 'STAR') ?>">
					<svg width="100" viewBox="0 0 100 90" xmlns="http://www.w3.org/2000/svg" version="1.1">
<?php
			if($average < $i and $i < $average + 1) {
				# 2 rectangles juxtaposés de couleur différente et clippés par l'étoile
				$ratio = $average + 1 - $i;
?>
						<rect clip-path="url(#filledStar)" x="5" width="90" height="90" />
						<rect clip-path="url(#filledStar)" x="5" width="<?= $ratio * 90 ?>" height="90" class="on" />
<?php
			} else {
					# 1 étoile de base
?>
						<use xlink:href="#star" />
<?php
			}

?>
					</svg>
				</li>
<?php
		}
	}

	private function getFilename($artId) {
		return self::WORKING_DIR . '/' . str_pad($artId, 4, '0', STR_PAD_LEFT) . '-' . $this->default_lang . '.xml';
	}

	private function getHash($value) {
		return hash('md4', $value);
	}

	static public function getIndexValue($values, $key, $default=0) {
		if(
			!isset($key) or
			!is_array($key) or
			!isset($values[$key[0]]) or
			!isset($values[$key[0]]['value'])
		) {
			return $default;
		}

		return $values[$key[0]]['value'];
	}

	public function getBallotsArt($artId) {
		$filename = $this->getFilename($artId);
		if(!file_exists($filename)) {
			return array(
				'sum'	=> 0,
				'ips'	=> array(),
			);
		}

		# Mise en place du parseur XML
		$datas = file_get_contents($filename);
		$parser = xml_parser_create(PLX_CHARSET);
		xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING,0);
		xml_parser_set_option($parser, XML_OPTION_SKIP_WHITE,0);
		xml_parse_into_struct($parser, $datas, $values, $iTags);
		xml_parser_free($parser);

		$shieldIps = self::getIndexValue($values, $iTags['used_ips']); # plxUtils::getValue($values[$iTags['used_ips'][0]]['value']),
		return array(
			'sum'	=> self::getIndexValue($values, $iTags['total_value']), # plxUtils::getValue($values[$iTags['total_value'][0]]['value']),
			'ips'	=> explode(PHP_EOL, trim($shieldIps)),
		);
	}

	public function saveBallotsArt($artId, $datas) {
		$filename = $this->getFilename($artId);
		ob_start();
?>
<rating>
	<total_value><?= round($datas['sum'], 5) ?></total_value>
	<used_ips>
<?= implode(PHP_EOL,$datas['ips']) . PHP_EOL ?>
	</used_ips>
</rating>
<?php
		file_put_contents($filename, self::XML_HEADER . ob_get_clean());
	}

	public function newBallotArt($idArt, $rating, $auth, $active) {
		$note = 0;
		$stars = '';
		if($auth != $this->getHash($_SESSION[__CLASS__])) {
			$status = 5;
		} else {
			$datas = $this->getBallotsArt($idArt);
			$ip = base_convert(ip2long($_SERVER['REMOTE_ADDR']), 10, 32); # Adresse IP convertie en un nombre en base 32
			if(in_array($ip, $datas['ips'])) {
				# Vote déjà enregistré pour cette addresse IP et cet article
				$status = 2;
			} else {
				if($rating< 1 or $rating > self::STARS_COUNT) {
					# notation invalide
					$satus = 4;
				} elseif(!$active) {
					# l'article n'est pas actif ou n'existe pas
					$status = 3;
				} else {
					$datas['sum'] += $rating / self::STARS_COUNT; # On peut changer le nombre d'étoiles en conservant la note
					$datas['ips'][] = $ip;
					sort($datas['ips']);
					$this->saveBallotsArt($idArt, $datas);
					$n = count($datas['ips']);
					$note = $datas['sum'] * self::STARS_COUNT / $n;
					$ballots = ($n > 1) ? count($datas['ips']) . ' ' . $this->getLang('BALLOTS') : $this->getLang('ONE_BALLOT');
					ob_start();
					$this->getStars($note);
					# Optimisation : suppression espaces inutiles, remplacement " par '.
					$stars = trim(
						str_replace('"', '\'',
							preg_replace('@>\s+<@', '><',
								preg_replace('@\s+@', ' ', ob_get_clean()) # Attention à viewbox dans <svg>
							)
						)
					);
					$status = 1;
				}
			}
		}

		# Réponse à la requête envoyée par XMLHttpRequest
		header('Cache-Control: no-cache');
		header('Pragma: nocache');
		header('Content-Type: application/json');
		echo json_encode(array(
			'id'		=> $idArt,
			'status'	=> $status,
			'text'		=> $this->getLang('STATUS-' . $status),
			'note'		=> sprintf(self::FORMAT_NUM, $note),
			'ballots'	=> !empty($ballots) ? $ballots : '',
			'stars'		=> $stars,
		), JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);

		exit;
	}

	/* ========== site hooks ======= */

	public function kzRating($params) {
		if(empty($this->secret_key)) {
			$this->secret_key = plxUtils::charAleatoire(16);
		}
		// https://css-tricks.com/clipping-masking-css/
		// https://www.sarasoueidan.com/blog/css-svg-clipping/
		// https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill-rule
		$plxShow = plxShow::getInstance();
		if(!in_array($plxShow->plxMotor->mode, array('article', 'home', 'categorie', 'tags', 'archives'))) {
			return;
		}

		$artId = $plxShow->artId();
		$open = ($this->limitPublished < $plxShow->plxMotor->plxRecord_arts->f('date')) ? ' open' : '';

		# On récupère les votes déjà enregistrés
		$rating = $this->getBallotsArt($artId);

		$ballots = count($rating['ips']);

		if(empty($open)) {
			if($ballots == 0) {
				# Période de vote terminée et aucune évalutation
				return;
			} else {
				$notice = $this->getLang('CLOSED');
			}
		} else {
			$notice = '';
		}

		if($ballots > 0) {
			$ip = base_convert(ip2long($_SERVER['REMOTE_ADDR']), 10, 32); # Adresse IP convertie en un nombre en base 32
			$hasVoted = in_array($ip, $rating['ips']);
			$open = '';
			$notice = $this->getLang('HAS_VOTED');
		}

		if(!empty($notice)) {
			$notice = ' - <em>' . $notice . '</em>';
		}

		# Pas de division par zéro
		$average = ($ballots > 0) ? $rating['sum'] * self::STARS_COUNT / $ballots : 0;

		switch($ballots) {
			case 0: $caption = $this->getLang('NO_BALLOT'); break;
			case 1: $caption = $this->getLang('ONE_BALLOT'); break;
			default: $caption = $ballots . ' ' . $this->getLang('BALLOTS');
		}
?>
		<div class="<?= __CLASS__ ?>-container">
			<ul class="<?= __CLASS__ . $open ?>" data-art="<?= $artId ?>">

<?php
		$this->getStars($average);
?>
			</ul>
			<div>
				<div><?php $this->lang('AVERAGE') ?> : <span class="note"><?php printf(self::FORMAT_NUM, $average) ?></span> / <?= self::STARS_COUNT ?></div>
				<div>( <span class="ballots"><?= $caption ?></span><?= $notice ?> )</div>
			</div>
		</div>
<?php
	}

	public function ThemeEndBody() {
		if(empty($this->secret_key)) {
			# Le hook kzRating n'a pas été utilisé
			return;
		}

		$_SESSION[__CLASS__] = $this->secret_key;
		$hash = $this->getHash($_SESSION[__CLASS__]);
?>
	<svg width="0" viewBox="0 0 100 90" xmlns="http://www.w3.org/2000/svg" version="1.1">
		<defs>
			<path d="M50,0 21,90 98,35 2,35 79,90z" id="star" stroke="none" />
		</defs>
		<clipPath id="filledStar">
			<use xlink:href="#star" clip-rule="nonzero" />
		</clipPath>
	</svg>
	<div class="<?= __CLASS__ ?>-spinner">
		<div></div>
		<div></div>
	</div>
	<script src="<?= PLX_PLUGINS . __CLASS__ . '/' . __CLASS__ . '-site.js' ?>" data-id="<?= $hash ?>" data-path="<?= dirname($_SERVER['SCRIPT_NAME']) ?>"></script>
<?php
	}

	public function plxMotorPreChauffageBegin() {
		echo self::BEGIN_CODE;
?>
if(!empty($this->get) and preg_match('<?= self::QUERY_PATTERN ?>', $this->get, $matches)) {
	$kzPlugin = $this->plxPlugins->aPlugins['<?= __CLASS__ ?>'];
	$kzId = $matches[1];
	$kzPlugin->newBallotArt($kzId, $matches[3], $matches[4], array_key_exists(str_pad($kzId, 4, '0', STR_PAD_LEFT), $this->activeArts));
	return true;
}

return false;
<?php
		echo self::END_CODE;
	}

	/* ========== admin hooks ======= */

	public function AdminIndexFoot() {
		list($author, $repo) = explode('/', self::IP2LOCATION_PLUGIN_GITHUB_REPO);
		$_SESSION[__CLASS__ . '-dbPlugin'] = $repo;
		$_SESSION[__CLASS__ . '-admin'] = plxUtils::charAleatoire();
?>
<div class="button in-action-bar kzRating-link">
	<a href="<?= PLX_PLUGINS . __CLASS__ ?>/<?= __CLASS__?>-ips.php" target="_blank"><?php $this->lang('BALLOTS'); ?></a>
</div>
<?php
	}

	/*
	 * Localise l'adresse IP du commentaire si la base de données IP2LOCATION est installée.
	 * */
	public function AdminCommentFoot() {
		list($author, $repo) = explode('/', self::IP2LOCATION_PLUGIN_GITHUB_REPO);
		if(!is_dir(PLX_PLUGINS . __CLASS__ . '/' . $repo)) {
			return;
		}
?>
	<script src="<?= PLX_PLUGINS . __CLASS__ . '/' . __CLASS__ . '-admin.js' ?>" data-path="<?= dirname($_SERVER['SCRIPT_NAME']) ?>"></script>
<?php
	}
}
