#!/usr/bin/env php
<?php

/*
 * Génére les fichiers d'évaluation pour les articles de façon aléatoire.
 * Les adresses IP sont extraites de la base de données IP2LOCATION et localisées
 * par défaut aux pays francophones.
 * */

const PLX_ROOT = '';
const PLX_CHARSET = 'utf-8';
const XML_HEADER = '<?xml version="1.0" encoding="' . PLX_CHARSET . '"?>' . PHP_EOL;

# https://fr.wikipedia.org/wiki/Liste_des_pays_ayant_le_fran%C3%A7ais_pour_langue_officielle
const FRENCH_SPEAKING_COUNTRIES = array( # population en millions d'habitants
	'CD'	=> 81, # Répubique Démocratique du Congo
	'FR'	=> 67, #France
	'CA'	=> 36, # Canada
	'MG'	=> 26, # madagascar
	'CI'	=> 24, # Côte d'Ivoire
	'CM'	=> 24, # Cameroun
	'NE'	=> 21, # Niger
	'BF'	=> 19, # Burkina Faso
	'ML'	=> 19, # Mali
	'SN'	=> 16, # Sénégal
	'GN'	=> 13, # Guinée
	'RW'	=> 12, # Rwanda
	'BE'	=> 11, # Belgique
	'BJ'	=> 11, # Bénin
	'HT'	=> 11, # Haïti
	'BU'	=> 11, # Burundi
	'CH'	=>  8, # Suisse
);

# On récupère le dossier où seront stockés les fichiers d'évaluation
include '../../../config.php';
const TARGET = PLX_ROOT . PLX_CONFIG_PATH . 'plugins/kzRating/';

if(!is_dir(TARGET)) {
	mkdir(TARGET);
}

# On récupère des adresses IP pour la zone géographique choisie. Pays francophones par défaut.
# Sinon codes ISO des pays séparés par un espace dans une chaine de caractères.
include 'MyDatabase';
$db = new MyDatabase();
$resp = $db->getIpsCountry(implode(' ', array_keys(FRENCH_SPEAKING_COUNTRIES)));

if(empty($resp)) {
	echo 'No record found.' . PHP_EOL;
	exit;
}

$recordsCnt = count($resp);
echo $recordsCnt . ' records.' . PHP_EOL;

$visitors = 0;
for($artId=1; $artId<=80; $artId++) {
		$sum = 0;
		$polling = array();

		# On boucle sur chaque visiteur.
		for($ballot=0, $maxBallots=mt_rand(10, 200); $ballot<$maxBallots; $ballot++) {
				$i = mt_rand(0, $recordsCnt);
				list($ipFrom, $ipTo, $country) = $resp[$i]; // long integers for $ipFrom and $ipTo
				# convert ip to base 32
				$polling[] = base_convert(mt_rand($ipFrom, $ipTo), 10, 32);
				# evaluation
				$m = mt_rand(2, 9);
				$sum += mt_rand(mt_rand(1, $m), mt_rand($m, 10)) / 10;
		}
		sort($polling);
		$visitors += count($polling);
		$filename = TARGET . str_pad($artId, 4, '0', STR_PAD_LEFT) . '-fr.xml';

		# On génère le fichier d'évaluations pour cette article.
		ob_start();
?>
<rating>
	<total_value><?= $sum ?></total_value>
	<used_ips>
<?= implode(PHP_EOL, $polling) . PHP_EOL ?>
	</used_ips>
</rating>
<?php
		file_put_contents($filename, XML_HEADER . ob_get_clean());
		echo $filename . ' : ' . count($polling) . ' ballots' . PHP_EOL;
}

echo PHP_EOL . $visitors . ' visitors' . PHP_EOL;
